library(Rfast)
library(bnlearn)
library(pcalg)
library(pchc)


data(alarm)
modelstring = paste0("[HIST|LVF][CVP|LVV][PCWP|LVV][HYP][LVV|HYP:LVF][LVF]",
  "[STKV|HYP:LVF][ERLO][HRBP|ERLO:HR][HREK|ERCA:HR][ERCA][HRSA|ERCA:HR][ANES]",
  "[APL][TPR|APL][ECO2|ACO2:VLNG][KINK][MINV|INT:VLNG][FIO2][PVS|FIO2:VALV]",
  "[SAO2|PVS:SHNT][PAP|PMB][PMB][SHNT|INT:PMB][INT][PRSS|INT:KINK:VTUB][DISC]",
  "[MVS][VMCH|MVS][VTUB|DISC:VMCH][VLNG|INT:KINK:VTUB][VALV|INT:VLNG]",
  "[ACO2|VALV][CCHL|ACO2:ANES:SAO2:TPR][HR|CCHL][CO|HR:STKV][BP|CO:TPR]")
dag <- model2network(modelstring)
g1 <- dag2essgraph( as.graphNEL(dag) )

png(filename = paste("alarm.png", sep = ""), width = 6000, height = 5000, res = 700)

graphviz.plot(dag, shape = "ellipse")

dev.off()


n <- c(1000, 5000, 1e+4, 3e+4, 5e+4, 1e+5, 3e+5, 5e+5, 1e+6, 3e+6, 5e+6)
R <- 50
shdpc <- numeric( length(n) )
names(shdpc) <- paste("n=", n, sep = "")
shdfed <- shdmm1 <- shdmm2 <- shdpc  ## SHD
timefed <- timepc <- timemm1 <- timemm2 <- shdpc  ## times
ntestsfed <- ntestspc <- ntestsmm1 <- ntestsmm2 <- shdpc  ## number of tests
skeltimefed <- skeltimepc <- skeltimemm2 <- shdpc


  for ( i in 1:length(n) ) {  ## sample size
    t1 <- t2 <- t3 <- t4 <- shd1 <- shd2 <- shd3 <- shd4 <- test1 <- test2 <- 
    test3 <- test4 <- st2 <- st3 <- st4 <- numeric(R)
    for (vim in 1:R) {
      x <- bnlearn::rbn(dag, n = n[i], data = alarm)
      y <- x
      for (j in 1:37)  y[, j] <- as.numeric(x[, j]) - 1
      y <- as.matrix(y)
      ind <- which( is.na( rowsums(y) ) )
      if ( length(ind)> 0) {
        x <- x[-ind, ]
        y <- y[-ind, ]
      }
      mod <- system.time( a1 <- bnlearn::mmhc( x, restrict.args = list(alpha = 0.05, test = "mi"), 
                                maximize.args = list(score = "bde") ) )
      a2 <- pchc::mmhc(y, alpha = 0.05, method = "cat", score = "bde")
      a3 <- pchc::pchc(y, alpha = 0.05, method = "cat", score = "bde")
	a4 <- pchc::fedhc(y, alpha = 0.05, method = "cat", score = "bde") 
	
      ## performance evaluation
      b1 <- as( bnlearn::amat(a1), "graphNEL" )   
      b2 <- as( bnlearn::amat(a2$dag), "graphNEL" )
      b3 <- as( bnlearn::amat(a3$dag), "graphNEL" )
      b4 <- as( bnlearn::amat(a4$dag), "graphNEL" )

      shd1[vim] <- pcalg::shd(b1, g1)   ;   shd2[vim] <- pcalg::shd(b2, g1)
      shd3[vim] <- pcalg::shd(b3, g1)   ;   shd4[vim] <- pcalg::shd(b4, g1)

      test1[vim] <- a1$learning$ntests    ;    test2[vim] <- sum(a2$ini$n.tests)
      test3[vim] <- sum(a3$ini$n.tests)   ;    test4[vim] <- sum(a4$ini$n.tests)

      t1[vim] <- mod[3]       ;    t2[vim] <- a2$runtime  ## runtimes
      t3[vim] <- a3$runtime   ;    t4[vim] <- a4$runtime
 	st2[vim] <- a2$ini$runtime  ;   st3[vim] <- a3$ini$runtime   ;   st4[vim] <- a4$ini$runtime
   
      save.image("alarm.RData")

    }  ## end for (vim in 1:R) {

    shdmm1[i] <- mean(shd1)
    shdmm2[i] <- mean(shd2)
    shdpc[i] <- mean(shd3)
    shdfed[i] <- mean(shd4)

    ntestsmm1[i] <- mean(test1)
    ntestsmm2[i] <- mean(test2)
    ntestspc[i] <- mean(test3)
    ntestsfed[i] <- mean(test4)    

    timemm1[i] <- mean(t1)
    timemm2[i] <- mean(t2)	  
    timepc[i] <- mean(t3)
    timefed[i] <- mean(t4)

    skeltimemm2[i] <- mean(st2)
    skeltimepc[i] <- mean(st3)
    skeltimefed[i] <- mean(st4)

    y <- NULL
    x <- NULL
    save.image("alarm.RData")
  }  ##  end  for ( i in 1:length(n) ) {




### SHD
png(filename = paste("shd_alarm.png", sep = ""), width = 6000, height = 5000, res = 700)

plot(log(n), shdfed, type = "b", pch = 19, lwd = 2, col = 3, main = "Alarm",
ylim = c(min(shdfed, shdpc, shdmm1, shdmm2), max(shdfed, shdpc, shdmm1, shdmm2) ), 
xlab = "Sample size in log-scale", ylab = "SHD", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(min(log(n)), max(log(n)), length = 100)
polygon( c(xx[1] - 5, xx, xx[100] + 5), 
c( min(shdfed, shdpc, shdmm1, shdmm2) - 5, rep(max(shdfed, shdpc, shdmm1, shdmm2) + 5, 100), 
min(shdpc, shdpc, shdmm1, shdmm2) - 5 ), col = "lightgrey" )

abline(v = seq(4, 16, by = 2), col = "white", lwd = 2)
abline(h = seq(20, 42, by = 1), col = "white", lwd = 2)
lines(log(n), shdfed, col = 3, lwd = 2)
points(log(n), shdfed, pch = 19, col = 3, cex = 1.2)
lines(log(n), shdpc, col = 2, lwd = 2)
points(log(n), shdpc, pch = 18, col = 2, cex = 1.2)
lines(log(n), shdmm1, col = 1, lwd = 2)
points(log(n), shdmm1, pch = 17, col = 1, cex = 1.2)
lines(log(n), shdmm2, col = 4, lwd = 2)
points(log(n), shdmm2, pch = 16, col = 4, cex = 1.2)

legend("bottomleft", c("FEDHC", "PCHC", "MMHC-1", "MMHC-2"), col = c(3, 2, 1, 4), 
pch = c(19, 18, 17, 16), lwd = rep(2, 4), cex = 1.2 , text.col = c(3, 2, 1, 4) )

dev.off()


png(filename = paste("ntests_alarm.png", sep = ""), width = 6000, height = 5000, res = 700)

plot(log(n), ntestsfed, type = "b", pch = 19, lwd = 2, col = 3, main = "Alarm",
ylim = c( min(ntestsfed, ntestspc, ntestsmm1, ntestsmm2), max(ntestsfed, ntestspc, ntestsmm1, ntestsmm2) ), 
xlab = "Sample size in log-scale", ylab = "Number of tests", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(min(log(n)), max(log(n)), length = 100)
polygon( c(xx[1] - 7000, xx, xx[100] + 7000), 
c( min(ntestsfed, ntestspc, ntestsmm1, ntestsmm2) - 7000, rep(max(ntestsfed, ntestspc, ntestsmm1, ntestsmm2) + 7000, 100), 
min(ntestsfed, ntestspc, ntestsmm1, ntestsmm2) - 7000 ), col = "lightgrey" )

abline(v = seq(4, 16, by = 2), col = "white", lwd = 2)
abline(h = seq(0, 170000, 10000), col = "white", lwd = 2)
lines(log(n), ntestsfed, col = 3, lwd = 2)
points(log(n), ntestsfed, pch = 19, col = 3, cex = 1.2)
lines(log(n), ntestspc, col = 2, lwd = 2)
points(log(n), ntestspc, pch = 18, col = 2, cex = 1.2)
lines(log(n), ntestsmm1, col = 1, lwd = 2)
points(log(n), ntestsmm1, pch = 17, col = 1, cex = 1.2)
lines(log(n), ntestsmm2, col = 4, lwd = 2)
points(log(n), ntestsmm2, pch = 16, col = 4, cex = 1.2)

legend("topleft", c("FEDHC", "PCHC", "MMHC-1", "MMHC-2"), col = c(3, 2, 1, 4), 
pch = c(19, 18, 17, 16), lwd = rep(2, 4), cex = 1.2 , text.col = c(3, 2, 1, 4) )

dev.off()


png(filename = paste("skeltime_alarm.png", sep = ""), width = 6000, height = 5000, res = 700)

plot( log( n[-c(1:2)] ), log( skeltimefed[-c(1:2)] ), type = "b", pch = 19, lwd = 2, col = 3, main = "Alarm",
ylim = c( min( log( skeltimefed[-c(1:2)] ), log( skeltimepc[-c(1:2)] ), log( skeltimemm2[-c(1:2)] ) ), 
max( log( skeltimefed[-c(1:2)] ), log( skeltimepc[-c(1:2)] ), log( skeltimemm2[-c(1:2)] ) ) ), 
xlab = "Sample size in log-scale", ylab = "Computational cost in log seconds", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq( min( log(n[-c(1:2)] ) ), max( log(n[-c(1:2)] ) ), length = 100)
polygon( c(xx[1] - 500, xx, xx[100] + 500), 
c( min( log( skeltimefed[-c(1:2)] ), log( skeltimepc[-c(1:2)] ), log( skeltimemm2[-c(1:2)] ) ) - 1200, 
rep( max( log( skeltimefed[-c(1:2)] ), log( skeltimepc[-c(1:2)] ), log( skeltimemm2[-c(1:2)] ) ) + 1200, 100), 
max( log( skeltimefed[-c(1:2)] ), log( skeltimepc[-c(1:2)] ), log( skeltimemm2[-c(1:2)] ) ) - 1200 ), col = "lightgrey" )

abline(v = seq(4, 18, by = 1), col = "white", lwd = 2)
abline(h = seq(-4, 6, 1), col = "white", lwd = 2)
lines( log( n[-c(1:2)] ), log( skeltimefed[-c(1:2)] ), col = 3, lwd = 2)
points( log( n[-c(1:2)] ), log( skeltimefed[-c(1:2)] ), pch = 19, col = 3, cex = 1.2)
lines( log( n[-c(1:2)] ), log( skeltimepc[-c(1:2)] ), col = 2, lwd = 2)
points( log( n[-c(1:2)] ), log( skeltimepc[-c(1:2)] ), pch = 18, col = 2, cex = 1.2)
lines( log( n[-c(1:2)] ), log( skeltimemm2[-c(1:2)] ), col = 1, lwd = 2)
points( log( n[-c(1:2)] ), log( skeltimemm2[-c(1:2)] ), pch = 17, col = 1, cex = 1.2)

legend("topleft", c("FEDHC", "PCHC", "MMHC-2"), col = c(3:1), 
pch = c(19, 18, 17), lwd = rep(2, 3), cex = 1.2 , text.col = c(3:1) )

dev.off()



png(filename = paste("time_alarm.png", sep = ""), width = 6000, height = 5000, res = 700)

plot( log( n[-c(1:2)] ), log( timefed[-c(1:2)] ), type = "b", pch = 19, lwd = 2, col = 3, main = "Alarm",
ylim = c( min( log( timefed[-c(1:2)] ), log( timepc[-c(1:2)] ), log( timemm2[-c(1:2)] ) ), 
max( log( timefed[-c(1:2)] ), log( timepc[-c(1:2)] ), log( timemm2[-c(1:2)] ) ) ), 
xlab = "Sample size in log-scale", ylab = "Computational cost in log seconds", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq( min( log(n[-c(1:2)] ) ), max( log(n[-c(1:2)] ) ), length = 100)
polygon( c(xx[1] - 500, xx, xx[100] + 500), 
c( min( log( timefed[-c(1:2)] ), log( timepc[-c(1:2)] ), log( timemm2[-c(1:2)] ) ) - 1200, 
rep( max( log( timefed[-c(1:2)] ), log( timepc[-c(1:2)] ), log( timemm2[-c(1:2)] ) ) + 1200, 100), 
max( log( timefed[-c(1:2)] ), log( timepc[-c(1:2)] ), log( timemm2[-c(1:2)] ) ) - 1200 ), col = "lightgrey" )

abline(v = seq(4, 18, by = 1), col = "white", lwd = 2)
abline(h = seq(-4, 6, 1), col = "white", lwd = 2)
lines( log( n[-c(1:2)] ), log( timefed[-c(1:2)] ), col = 3, lwd = 2)
points( log( n[-c(1:2)] ), log( timefed[-c(1:2)] ), pch = 19, col = 3, cex = 1.2)
lines( log( n[-c(1:2)] ), log( timepc[-c(1:2)] ), col = 2, lwd = 2)
points( log( n[-c(1:2)] ), log( timepc[-c(1:2)] ), pch = 18, col = 2, cex = 1.2)
lines( log( n[-c(1:2)] ), log( timemm2[-c(1:2)] ), col = 1, lwd = 2)
points( log( n[-c(1:2)] ), log( timemm2[-c(1:2)] ), pch = 17, col = 1, cex = 1.2)

legend("topleft", c("FEDHC", "PCHC", "MMHC-2"), col = c(3:1), 
pch = c(19, 18, 17), lwd = rep(2, 3), cex = 1.2 , text.col = c(3:1) )

dev.off()