library(Rfast)
library(pchc)

###################
###### 3 Neighbours
###################

n <- c(100, 500, 1000, 5000, 1e+4, 3e+4, 5e+4, 1e+5, 3e+5, 5e+5, 1e+6, 3e+6, 5e+6)

R <- 50
p <- c(20, 30, 50)
shdfedrob <- matrix(nrow = length(n), ncol = length(p) )
colnames(shdfedrob) <- paste("p=", p, sep = "")
rownames(shdfedrob) <- paste("n=", n, sep = "")
shdfed <- shdfedrob  ## SHD
timefed <- timefedrob <- shdfedrob ## times
ntestsfed <- ntestsfedrob <- shdfedrob ## number of tests
skeltimefed <- skeltimefedrob <- shdfedrob

  for ( i in 1:length(n) ) {  ## sample size
    for ( j in 1:length(p) ) { ## number of variables

      st1 <- st2 <- t1 <- t2 <- shd1 <- shd2 <- test1 <- test2 <- numeric(R)
      for (vim in 1:R) {
        m <- MXM::rdag2(n = n[i], p = p[j], nei = 3)
        G1 <- m$G
        g1 <- pcalg::dag2essgraph( as(G1, "graphNEL") )  ## turn the true DAG to an essential DAG
        x <- m$x ## data
        ## run the algorigthms
	  a1 <- pchc::fedhc(x, alpha = 0.05)
        a2 <- try( pchc::fedhc(x, alpha = 0.05, robust = TRUE), silent = TRUE )
        while ( !identical( class(a2$dag), "bn" ) ) {
          a2 <- try( pchc::fedhc(x, alpha = 0.05, robust = TRUE), silent = TRUE )
        }
        ## performance evaluation
	  st1[vim] <- a1$ini$runtime   ;    st2[vim] <- a2$ini$runtime
        t1[vim] <- a1$runtime   ;    t2[vim] <- a2$runtime
	  b1 <- as( bnlearn::amat(a1$dag), "graphNEL" )
	  b2 <- as( bnlearn::amat(a2$dag), "graphNEL" )
        shd1[vim] <- pcalg::shd(b1, g1)   ;  shd2[vim] <-  pcalg::shd(b2, g1)  
        test1[vim] <- a1$ini$n.tests   ;   test2[vim] <- sum(a2$ini$n.tests)   
      }  ## end for (vim in 1:R) {

	skeltimefed[i, j] <- mean(st1)
      skeltimefedrob[i, j] <- mean(st2)
	  
      timefed[i, j] <- mean(t1)
      timefedrob[i, j] <- mean(t2)

      shdfed[i, j] <- mean(shd1)
      shdfedrob[i, j] <- mean(shd2)

      ntestsfed[i, j] <- mean(test1)
      ntestsfedrob[i, j] <- mean(test2)
	  
      x <- NULL
      save.image("robust_3_sims.RData")
    }  ##  end  for ( j in 1:length(p) ) {
  }  ##  end  for ( i in 1:length(n) ) {



### SHD
png(filename = paste("3_20_robust.png", sep = ""), width = 6000, height = 5000, res = 700)

rat1 <- shdfed[1:9, 1]/shdfedrob[1:9, 1]
rat2 <- timefed[1:9, 1]/timefedrob[1:9, 1]

plot(log(n[1:9]), rat1, type = "b", pch = 19, lwd = 2, col = 3, main = "20 variables",
ylim = c( min(rat1, rat2), max(rat1, rat2) ),  
xlab = "Sample size in log-scale", ylab = "Ratio", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(min(log(n[1:9])), max(log(n[1:9])), length = 100)
polygon( c(xx[1] - 20, xx, xx[100] + 20), 
c( min(rat1, rat2) - 4,  rep(max(rat1, rat2) + 4, 100), 
min(rat1, rat2) - 4 ), col = "lightgrey" )

abline(v = seq(4, 16, by = 2), col = "white", lwd = 2)
abline(h = seq(0, 1, 0.1), col = "white", lwd = 2)

lines(log(n[1:9]), rat1, col = 3, lwd = 2)
points(log(n[1:9]), rat1, pch = 19, col = 3, cex = 1.2)
lines(log(n[1:9]), rat2, col = 4, lwd = 2)
points(log(n[1:9]), rat2, pch = 18, col = 4, cex = 1.2)

legend("left", c("Ratio of SHDs", "Ratio of computational costs"), col = c(3, 4), 
pch = c(19, 18), lwd = c(2, 2), cex = 1.2 , text.col = c(3, 4) )

dev.off()

### SHD
png(filename = paste("3_30_robust.png", sep = ""), width = 6000, height = 5000, res = 700)

rat1 <- shdfed[1:9, 2]/shdfedrob[1:9, 2]
rat2 <- timefed[1:9, 2]/timefedrob[1:9, 2]

plot(log(n[1:9]), rat1, type = "b", pch = 19, lwd = 2, col = 3, main = "30 variables",
ylim = c( min(rat1, rat2), max(rat1, rat2) ),  
xlab = "Sample size in log-scale", ylab = "Ratio", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(min(log(n[1:9])), max(log(n[1:9])), length = 100)
polygon( c(xx[1] - 20, xx, xx[100] + 20), 
c( min(rat1, rat2) - 4,  rep(max(rat1, rat2) + 4, 100), 
min(rat1, rat2) - 4 ), col = "lightgrey" )

abline(v = seq(4, 14, by = 2), col = "white", lwd = 2)
abline(h = seq(0, 1, 0.1), col = "white", lwd = 2)

lines(log(n[1:9]), rat1, col = 3, lwd = 2)
points(log(n[1:9]), rat1, pch = 19, col = 3, cex = 1.2)
lines(log(n[1:9]), rat2, col = 4, lwd = 2)
points(log(n[1:9]), rat2, pch = 18, col = 4, cex = 1.2)

legend("left", c("Ratio of SHDs", "Ratio of computational costs"), col = c(3, 4), 
pch = c(19, 18), lwd = c(2, 2), cex = 1.2 , text.col = c(3, 4) )

dev.off()


### SHD
png(filename = paste("3_50_robust.png", sep = ""), width = 6000, height = 5000, res = 700)

rat1 <- shdfed[1:9, 3]/shdfedrob[1:9, 3]
rat2 <- timefed[1:9, 3]/timefedrob[1:9, 3]

plot(log(n[1:9]), rat1, type = "b", pch = 19, lwd = 2, col = 3, main = "50 variables",
ylim = c( min(rat1, rat2), max(rat1, rat2) ),  
xlab = "Sample size in log-scale", ylab = "Ratio", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(min(log(n[1:9])), max(log(n[1:9])), length = 100)
polygon( c(xx[1] - 20, xx, xx[100] + 20), 
c( min(rat1, rat2) - 4,  rep(max(rat1, rat2) + 4, 100), 
min(rat1, rat2) - 4 ), col = "lightgrey" )

abline(v = seq(4, 14, by = 2), col = "white", lwd = 2)
abline(h = seq(0, 1, 0.1), col = "white", lwd = 2)

lines(log(n[1:9]), rat1, col = 3, lwd = 2)
points(log(n[1:9]), rat1, pch = 19, col = 3, cex = 1.2)
lines(log(n[1:9]), rat2, col = 4, lwd = 2)
points(log(n[1:9]), rat2, pch = 18, col = 4, cex = 1.2)

legend("left", c("Ratio of SHDs", "Ratio of computational costs"), col = c(3, 4), 
pch = c(19, 18), lwd = c(2, 2), cex = 1.2 , text.col = c(3, 4) )

dev.off()





n <- c(500, 1000, 5000, 1e+4, 3e+4, 5e+4, 1e+5, 3e+5, 5e+5, 1e+6, 3e+6, 5e+6)

R <- 50
p <- 100
shdfedrob <- matrix(nrow = length(n), ncol = length(p) )
colnames(shdfedrob) <- paste("p=", p, sep = "")
rownames(shdfedrob) <- paste("n=", n, sep = "")
shdfed <- shdfedrob  ## SHD
timefed <- timefedrob <- shdfedrob ## times
ntestsfed <- ntestsfedrob <- shdfedrob ## number of tests
skeltimefed <- skeltimefedrob <- shdfedrob

  for ( i in 1:length(n) ) {  ## sample size
    for ( j in length(p) ) { ## number of variables

      st1 <- st2 <- t1 <- t2 <- shd1 <- shd2 <- test1 <- test2 <- numeric(R)
      for (vim in 1:R) {
        m <- MXM::rdag2(n = n[i], p = p[j], nei = 3)
        G1 <- m$G
        g1 <- pcalg::dag2essgraph( as(G1, "graphNEL") )  ## turn the true DAG to an essential DAG
        x <- m$x ## data
        ## run the algorigthms
	  a1 <- pchc::fedhc(x, alpha = 0.05)
        a2 <- try( pchc::fedhc(x, alpha = 0.05, robust = TRUE), silent = TRUE )
        while ( !identical( class(a2$dag), "bn" ) ) {
          a2 <- try( pchc::fedhc(x, alpha = 0.05, robust = TRUE), silent = TRUE )
        }
        ## performance evaluation
	  st1[vim] <- a1$ini$runtime   ;    st2[vim] <- a2$ini$runtime
        t1[vim] <- a1$runtime   ;    t2[vim] <- a2$runtime
	  b1 <- as( bnlearn::amat(a1$dag), "graphNEL" )
	  b2 <- as( bnlearn::amat(a2$dag), "graphNEL" )
        shd1[vim] <- pcalg::shd(b1, g1)   ;  shd2[vim] <-  pcalg::shd(b2, g1)  
        test1[vim] <- a1$ini$n.tests   ;   test2[vim] <- sum(a2$ini$n.tests)   
      }  ## end for (vim in 1:R) {

	skeltimefed[i, j] <- mean(st1)
      skeltimefedrob[i, j] <- mean(st2)
	  
      timefed[i, j] <- mean(t1)
      timefedrob[i, j] <- mean(t2)

      shdfed[i, j] <- mean(shd1)
      shdfedrob[i, j] <- mean(shd2)

      ntestsfed[i, j] <- mean(test1)
      ntestsfedrob[i, j] <- mean(test2)
	  
      x <- NULL
      save.image("robust_3_sims_100.RData")
    }  ##  end  for ( j in 1:length(p) ) {
  }  ##  end  for ( i in 1:length(n) ) {



### SHD
png(filename = paste("3_100_robust.png", sep = ""), width = 6000, height = 5000, res = 700)

rat1 <- shdfed[1:12, 1]/shdfedrob[1:12, 1]
rat2 <- timefed[1:12, 1]/timefedrob[1:12, 1]

plot(log(n[1:12]), rat1, type = "b", pch = 19, lwd = 2, col = 3, main = "100 variables",
ylim = c( min(rat1, rat2), max(rat1, rat2) ),  
xlab = "Sample size in log-scale", ylab = "Ratio", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(min(log(n[1:12])), max(log(n[1:12])), length = 100)
polygon( c(xx[1] - 20, xx, xx[100] + 20), 
c( min(rat1, rat2) - 4,  rep(max(rat1, rat2) + 4, 100), 
min(rat1, rat2) - 4 ), col = "lightgrey" )

abline(v = seq(4, 14, by = 2), col = "white", lwd = 2)
abline(h = seq(0, 1, 0.1), col = "white", lwd = 2)

lines(log(n[1:12]), rat1, col = 3, lwd = 2)
points(log(n[1:12]), rat1, pch = 19, col = 3, cex = 1.2)
lines(log(n[1:12]), rat2, col = "blue", lwd = 2)
points(log(n[1:12]), rat2, pch = 18, col = "blue", cex = 1.2)

legend("left", c("Ratio of SHDs", "Ratio of computational costs"), col = c(3, "blue"), 
pch = c(19, 18), lwd = c(2, 2), cex = 1.2 , text.col = c(3, "blue") )

dev.off()






