library(MXM)
library(Rfast)
library(bnlearn)
library(pcalg)
library(pchc)



###################
###### 3 Neighbours
###################

n <- c(1000, 5000,  1e+4, 5e+4, 1e+5, 2e+5, 5e+5, 1e+6, 2e+6, 3e+6, 4e+6, 5e+6)
R <- 20
p <- c(20, 30, 50)
shdfed1 <- matrix(nrow = length(n), ncol = length(p) )
colnames(shdfed1) <- paste("p=", p, sep = "")
rownames(shdfed1) <- paste("n=", n, sep = "")
timefed1 <- shdfed1  ## times
timemm1 <- shdmm1 <- shdfed1

  for ( i in 1:length(n) ) {  ## sample size
    for ( j in 1:length(p) ) { ## number of variables
      t1 <- shd1 <- t2 <- shd2 <- numeric(R)
      for (vim in 1:R) {
        m <- MXM::rdag2(n = n[i], p = p[j], nei = 3)
        G1 <- m$G
        g1 <- pcalg::dag2essgraph( as(G1, "graphNEL") )  ## turn the true DAG to an essential DAG
        x <- m$x ## data
        a1 <- fedhc(x)
        a2 <- pchc::mmhc(x)
        t1[vim] <- a1$runtime[3]  ## runtimes
        t2[vim] <- a2$runtime[3]  ## runtimes
        b1 <- as( bnlearn::amat(a1$dag), "graphNEL" )
        b2 <- as( bnlearn::amat(a2$dag), "graphNEL" )
        shd1[vim] <-  pcalg::shd(b1, g1)
        shd2[vim] <-  pcalg::shd(b2, g1)
      }  ## end for (vim in 1:R) {
      timefed1[i, j] <- mean(t1)
      shdfed1[i, j] <- mean(shd1)
      timemm1[i, j] <- mean(t2)
      shdmm1[i, j] <- mean(shd2)
      save.image("time_fedhc.RData")
    }  ##  end  for ( j in 1:length(p) ) {
  }  ##  end  for ( i in 1:length(n) ) {


###################
###### 5 Neighbours
###################

n <- c(1000, 5000,  1e+4, 5e+4, 1e+5, 2e+5, 5e+5, 1e+6, 2e+6, 3e+6, 4e+6, 5e+6)
R <- 20
p <- c(20, 30, 50)
shdfed2 <- matrix(nrow = length(n), ncol = length(p) )
colnames(shdfed2) <- paste("p=", p, sep = "")
rownames(shdfed2) <- paste("n=", n, sep = "")
timefed2 <- shdfed2  ## times
timemm2 <- shdmm2 <- shdfed2

  for ( i in 1:length(n) ) {  ## sample size
    for ( j in 1:length(p) ) { ## number of variables
      t3 <- shd3 <- t4 <- shd4 <- numeric(R)
      for (vim in 1:R) {
        m <- MXM::rdag2(n = n[i], p = p[j], nei = 5)
        G2 <- m$G
        g2 <- pcalg::dag2essgraph( as(G2, "graphNEL") )  ## turn the true DAG to an essential DAG
        x <- m$x ## data
        a3 <- fedhc(x)
        a4 <- pchc::mmhc(x)
        t3[vim] <- a3$runtime[3]  ## runtimes
        t4[vim] <- a4$runtime[3]  ## runtimes
        b3 <- as( bnlearn::amat(a3$dag), "graphNEL" )
        b4 <- as( bnlearn::amat(a4$dag), "graphNEL" )
        shd3[vim] <-  pcalg::shd(b3, g2)
        shd4[vim] <-  pcalg::shd(b4, g2)
      }  ## end for (vim in 1:R) {
      timefed2[i, j] <- mean(t3)
      shdfed2[i, j] <- mean(shd3)
      timemm2[i, j] <- mean(t4)
      shdmm2[i, j] <- mean(shd4)
      save.image("time_fedhc.RData")
    }  ##  end  for ( j in 1:length(p) ) {
  }  ##  end  for ( i in 1:length(n) ) {





png(filename = paste("scalab1.png", sep = ""), width = 6000, height = 5000, res = 700)

plot(log(n), log(timepc1)[, 1], type = "b", pch = 18, lwd = 2, 
ylim = c( min(log(timepc1)), max(log(timepc1)) ), 
xlab = "Sample size in log-scale", ylab = "Time(seconds) in log-scale", 
cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)
for (i in 2:5) { 
  lines(log(n), log(timepc1)[, i], col = i, lwd = 2)
  points(log(n), log(timepc1)[, i], pch = 18, col = i, cex = 1.2)

}
abline(h = 0, col = "lightgrey", lwd = 3)
abline(h = seq(-4, 4, by = 2), col = "lightgrey", lwd = 2, lty = 2)
legend("topleft", c("p=10", "p=20", "p=30", "p=40", "p=50"), col = c(1:5), 
pch = rep(18, 5), lwd = rep(2, 5), cex = 1.2 )

dev.off()

png(filename = paste("scalab2.png", sep = ""), width = 6000, height = 5000, res = 700)

plot(log(n), log(timepc2)[, 1], type = "b", pch = 18, lwd = 2, 
ylim = c( min(log(timepc2)), max(log(timepc2)) ), 
xlab = "Sample size in log-scale", ylab = "Time(seconds) in log-scale", 
cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)
for (i in 2:5) { 
  lines(log(n), log(timepc2)[, i], col = i, lwd = 2)
  points(log(n), log(timepc2)[, i], pch = 18, col = i, cex = 1.2)

}
abline(h = 0, col = "lightgrey", lwd = 3)
abline(h = seq(-4, 4, by = 2), col = "lightgrey", lwd = 2, lty = 2)
legend("topleft", c("p=10", "p=20", "p=30", "p=40", "p=50"), col = c(1:5), 
pch = rep(18, 5), lwd = rep(2, 5), cex = 1.2 )

dev.off()
