library(Rfast)
library(pchc)

###################
###### 3 Neighbours
###################

n <- c(100, 500, 1000, 5000, 1e+4, 3e+4, 5e+4, 1e+5, 3e+5, 5e+5, 1e+6, 3e+6, 5e+6)

R <- 50
p <- c(20, 30, 50)
shdpc <- matrix(nrow = length(n), ncol = length(p) )
colnames(shdpc) <- paste("p=", p, sep = "")
rownames(shdpc) <- paste("n=", n, sep = "")
shdfed <- shdmm <- shdpc  ## SHD
timefed <- timepc <- timemm <- shdpc  ## times
ntestsfed <- ntestspc <- ntestsmm <- shdpc  ## number of tests
skeltimefed <- skeltimepc <- skeltimemm <- shdpc

  for ( i in 1:length(n) ) {  ## sample size
    for ( j in 1:length(p) ) { ## number of variables
      st1 <- st2 <- st3 <- t1 <- t2 <- t3 <- shd1 <- shd2 <- shd3 <- test1 <- test2 <- test3 <- numeric(R)
      for (vim in 1:R) {
        m <- MXM::rdag2(n = n[i], p = p[j], nei = 5)
        G1 <- m$G
        g1 <- pcalg::dag2essgraph( as(G1, "graphNEL") )  ## turn the true DAG to an essential DAG
        x <- m$x ## data
        ## run the algorigthms
        #mod <- system.time( a1 <- bnlearn::mmhc( as.data.frame(x), restrict.args = list(alpha = 0.05, test = "zf") ) )
	  a1 <- pchc::fedhc(x, alpha = 0.05) 
        a2 <- pchc::pchc(x, alpha = 0.05)
	  a3 <- pchc::mmhc(x, alpha = 0.05)
        ## performance evaluation
	  st1[vim] <- a1$ini$runtime   ;    st2[vim] <- a2$ini$runtime  ;   st3[vim] <- a3$ini$runtime  ## runtimes
        t1[vim] <- a1$runtime   ;    t2[vim] <- a2$runtime ;   t3[vim] <- a3$runtime  ## runtimes
	  b1 <- as( bnlearn::amat(a1$dag), "graphNEL" )
	  b2 <- as( bnlearn::amat(a2$dag), "graphNEL" )
        b3 <- as( bnlearn::amat(a3$dag), "graphNEL" )
        shd1[vim] <- pcalg::shd(b1, g1)   ;  shd2[vim] <-  pcalg::shd(b2, g1)   ;  shd3[vim] <-  pcalg::shd(b3, g1)
        test1[vim] <- a1$ini$n.tests   ;   test2[vim] <- sum(a2$ini$n.tests)   ;   test3[vim] <- a3$ini$n.tests
      }  ## end for (vim in 1:R) {
	skeltimefed[i, j] <- mean(st1)
      skeltimepc[i, j] <- mean(st2)
	skeltimemm[i, j] <- mean(st3)
	  
      timefed[i, j] <- mean(t1)
      timepc[i, j] <- mean(t2)
      timemm[i, j] <- mean(t3)

      shdfed[i, j] <- mean(shd1)
      shdpc[i, j] <- mean(shd2)
      shdmm[i, j] <- mean(shd3)

      ntestsfed[i, j] <- mean(test1)
      ntestspc[i, j] <- mean(test2)
	ntestsmm[i, j] <- mean(test3)
	
      x <- NULL  
      save.image("sims_5.RData")
    }  ##  end  for ( j in 1:length(p) ) {
  }  ##  end  for ( i in 1:length(n) ) {



### SHD
png(filename = paste("shd_5_20.png", sep = ""), width = 6000, height = 5000, res = 700)

plot(log(n), shdfed[, 1], type = "b", pch = 19, lwd = 2, col = 3, main = "20 variables",
ylim = c( min(shdfed[, 1], shdpc[, 1], shdmm[, 1]), max(shdfed[, 1], shdpc[, 1], shdmm[, 1]) ), 
xlab = "Sample size in log-scale", ylab = "SHD", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(min(log(n)), max(log(n)), length = 100)
polygon( c(xx[1] - 10, xx, xx[100] + 5), 
c( min(shdfed[, 1], shdpc[, 1], shdmm[, 1]) - 10, rep(max(shdfed[, 1], shdpc[, 1], shdmm[, 1]) + 5, 100), 
min(shdpc[, 1], shdpc[, 1], shdmm[, 1]) - 10), col = "lightgrey" )

abline(v = seq(4, 16, by = 2), col = "white", lwd = 2)
abline(h = seq(6, 40, 1), col = "white", lwd = 2)
lines(log(n), shdfed[, 1], col = 3, lwd = 2)
points(log(n), shdfed[, 1], pch = 19, col = 3, cex = 1.2)
lines(log(n), shdpc[, 1], col = 2, lwd = 2)
points(log(n), shdpc[, 1], pch = 18, col = 2, cex = 1.2)
lines(log(n), shdmm[, 1], col = 1, lwd = 2)
points(log(n), shdmm[, 1], pch = 17, col = 1, cex = 1.2)

legend("topright", c("FEDHC", "PCHC", "MMHC-2"), col = c(3:1), 
pch = c(19, 18, 17), lwd = rep(2, 3), cex = 1.2 , text.col = c(3:1) )

dev.off()


png(filename = paste("shd_5_30.png", sep = ""), width = 6000, height = 5000, res = 700)

plot(log(n), shdfed[, 2], type = "b", pch = 19, lwd = 2, col = 3, main = "30 variables",
ylim = c( min(shdfed[, 2], shdpc[, 2], shdmm[, 2]), max(shdfed[, 2], shdpc[, 2], shdmm[, 2]) ), 
xlab = "Sample size in log-scale", ylab = "SHD", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(min(log(n)), max(log(n)), length = 100)
polygon( c(xx[1] - 5, xx, xx[100] + 10), 
c(min(shdfed[, 2], shdpc[, 2], shdmm[, 2]) - 5, rep(max(shdfed[, 2], shdpc[, 2], shdmm[, 2]) + 10, 100), 
min(shdfed[, 2], shdpc[, 2], shdmm[, 2]) - 5 ), col = "lightgrey" )

abline(v = seq(4, 16, by = 2), col = "white", lwd = 2)
abline(h = seq(8, 52, 1), col = "white", lwd = 2)
lines(log(n), shdfed[, 2], col = 3, lwd = 2)
points(log(n), shdfed[, 2], pch = 19, col = 3, cex = 1.2)
lines(log(n), shdpc[, 2], col = 2, lwd = 2)
points(log(n), shdpc[, 2], pch = 18, col = 2, cex = 1.2)
lines(log(n), shdmm[, 2], col = 1, lwd = 2)
points(log(n), shdmm[, 2], pch = 17, col = 1, cex = 1.2)

legend("topright", c("FEDHC", "PCHC", "MMHC-2"), col = c(3:1), 
pch = c(19, 18, 17), lwd = rep(2, 3), cex = 1.2 , text.col = c(3:1) )

dev.off()


png(filename = paste("shd_5_50.png", sep = ""), width = 6000, height = 5000, res = 700)

plot(log(n), shdfed[, 3], type = "b", pch = 19, lwd = 2, col = 3, main = "50 variables",
ylim = c( min(shdfed[, 3], shdpc[, 3], shdmm[, 3]), max(shdfed[, 3], shdpc[, 2], shdmm[, 3]) ), 
xlab = "Sample size in log-scale", ylab = "SHD", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(min(log(n)), max(log(n)), length = 100)
polygon( c(xx[1] - 10, xx, xx[100] + 10), 
c(min(shdfed[, 3], shdpc[, 3], shdmm[, 3]) - 10, rep(max(shdfed[, 3], shdpc[, 3], shdmm[, 3]) + 10, 100), 
min(shdfed[, 3], shdpc[, 3], shdmm[, 3]) - 10 ), col = "lightgrey" )

abline(v = seq(4, 16, by = 2), col = "white", lwd = 2)
abline(h = seq(10, 90, 2), col = "white", lwd = 2)
lines(log(n), shdfed[, 3], col = 3, lwd = 2)
points(log(n), shdfed[, 3], pch = 19, col = 3, cex = 1.2)
lines(log(n), shdpc[, 3], col = 2, lwd = 2)
points(log(n), shdpc[, 3], pch = 18, col = 2, cex = 1.2)
lines(log(n), shdmm[, 3], col = 1, lwd = 2)
points(log(n), shdmm[, 3], pch = 17, col = 1, cex = 1.2)

legend("topright", c("FEDHC", "PCHC", "MMHC-2"), col = c(3:1), 
pch = c(19, 18, 17), lwd = rep(2, 3), cex = 1.2 , text.col = c(3:1) )

dev.off()


## Number of CI tests


png(filename = paste("ntests_5_20.png", sep = ""), width = 6000, height = 5000, res = 700)

plot(log(n), ntestsfed[, 1], type = "b", pch = 19, lwd = 2, col = 3, main = "20 variables",
ylim = c( min(ntestsfed[, 1], ntestspc[, 1], ntestsmm[, 1]), max(ntestsfed[, 1], ntestspc[, 1], ntestsmm[, 1]) ), 
xlab = "Sample size in log-scale", ylab = "Number of tests", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(min(log(n)), max(log(n)), length = 100)
polygon( c(xx[1] - 2000, xx, xx[100] + 2000), 
c( min(ntestsfed[, 1], ntestspc[, 1], ntestsmm[, 1]) - 2000, rep(max(ntestsfed[, 1], ntestspc[, 1], ntestsmm[, 1]) + 2000, 100), 
min(ntestspc[, 1], ntestspc[, 1], ntestsmm[, 1]) - 2000 ), col = "lightgrey" )

abline(v = seq(4, 16, by = 2), col = "white", lwd = 2)
abline(h = seq(0, 32000, 1000), col = "white", lwd = 2)
lines(log(n), ntestsfed[, 1], col = 3, lwd = 2)
points(log(n), ntestsfed[, 1], pch = 19, col = 3, cex = 1.2)
lines(log(n), ntestspc[, 1], col = 2, lwd = 2)
points(log(n), ntestspc[, 1], pch = 18, col = 2, cex = 1.2)
lines(log(n), ntestsmm[, 1], col = 1, lwd = 2)
points(log(n), ntestsmm[, 1], pch = 17, col = 1, cex = 1.2)

legend("topleft", c("FEDHC", "PCHC", "MMHC-2"), col = c(3:1), 
pch = c(19, 18, 17), lwd = rep(2, 3), cex = 1.2 , text.col = c(3:1) )

dev.off()


png(filename = paste("ntests_5_30.png", sep = ""), width = 6000, height = 5000, res = 700)

plot(log(n), ntestsfed[, 2], type = "b", pch = 19, lwd = 2, col = 3, main = "30 variables",
ylim = c( min(ntestsfed[, 2], ntestspc[, 2], ntestsmm[, 2]), max(ntestsfed[, 2], ntestspc[, 2], ntestsmm[, 2]) ), 
xlab = "Sample size in log-scale", ylab = "Number of tests", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(min(log(n)), max(log(n)), length = 100)
polygon( c(xx[1] - 5000, xx, xx[100] + 5000), 
c(min(ntestsfed[, 2], ntestspc[, 2], ntestsmm[, 2]) - 5000, 
rep(max(ntestsfed[, 2], ntestspc[, 2], ntestsmm[, 2]) + 5000, 100), 
min(ntestsfed[, 2], ntestspc[, 2], ntestsmm[, 2]) - 5000 ), col = "lightgrey" )

abline(v = seq(4, 16, by = 2), col = "white", lwd = 2)
abline(h = seq(0, 88000, 2000), col = "white", lwd = 2)
lines(log(n), ntestsfed[, 2], col = 3, lwd = 2)
points(log(n), ntestsfed[, 2], pch = 19, col = 3, cex = 1.2)
lines(log(n), ntestspc[, 2], col = 2, lwd = 2)
points(log(n), ntestspc[, 2], pch = 18, col = 2, cex = 1.2)
lines(log(n), ntestsmm[, 2], col = 1, lwd = 2)
points(log(n), ntestsmm[, 2], pch = 17, col = 1, cex = 1.2)

legend("topleft", c("FEDHC", "PCHC", "MMHC-2"), col = c(3:1), 
pch = c(19, 18, 17), lwd = rep(2, 3), cex = 1.2 , text.col = c(3:1) )

dev.off()


png(filename = paste("ntests_5_50.png", sep = ""), width = 6000, height = 5000, res = 700)

plot(log(n), ntestsfed[, 3], type = "b", pch = 19, lwd = 2, col = 3, main = "50 variables",
ylim = c( min(ntestsfed[, 3], ntestspc[, 3], ntestsmm[, 3]), max(ntestsfed[, 3], ntestspc[, 2], ntestsmm[, 3]) ), 
xlab = "Sample size in log-scale", ylab = "Number of tests", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(min(log(n)), max(log(n)), length = 100)
polygon( c(xx[1] - 20000, xx, xx[100] + 20000), 
c( min(ntestsfed[, 3], ntestspc[, 3], ntestsmm[, 3]) - 20000, 
rep(max(ntestsfed[, 3], ntestspc[, 3], ntestsmm[, 3]) + 20000, 100), 
min(ntestsfed[, 3], ntestspc[, 3], ntestsmm[, 3]) - 20000 ), col = "lightgrey" )

abline(v = seq(4, 16, by = 2), col = "white", lwd = 2)
abline(h = seq(0, 290000, 5000), col = "white", lwd = 2)
lines(log(n), ntestsfed[, 3], col = 3, lwd = 2)
points(log(n), ntestsfed[, 3], pch = 19, col = 3, cex = 1.2)
lines(log(n), ntestspc[, 3], col = 2, lwd = 2)
points(log(n), ntestspc[, 3], pch = 18, col = 2, cex = 1.2)
lines(log(n), ntestsmm[, 3], col = 1, lwd = 2)
points(log(n), ntestsmm[, 3], pch = 17, col = 1, cex = 1.2)

legend("topleft", c("FEDHC", "PCHC", "MMHC-2"), col = c(3:1), 
pch = c(19, 18, 17), lwd = rep(2, 3), cex = 1.2 , text.col = c(3:1) )

dev.off()


png(filename = paste("skeltime_5_50.png", sep = ""), width = 6000, height = 5000, res = 700)

plot( log( n[-c(1:2)] ), log( skeltimefed[-c(1:2), 3] ), type = "b", pch = 19, lwd = 2, col = 3, main = "50 variables",
ylim = c( min( log( skeltimefed[-c(1:2), 3] ), log( skeltimepc[-c(1:2), 3] ), log( skeltimemm[-c(1:2), 3] ) ), 
max( log( skeltimefed[-c(1:2), 3] ), log( skeltimepc[-c(1:2), 3] ), log( skeltimemm[-c(1:2), 3] ) ) ), 
xlab = "Sample size in log-scale", ylab = "Computational cost in log seconds", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq( min( log(n[-c(1:2)] ) ), max( log(n[-c(1:2)] ) ), length = 100)
polygon( c(xx[1] - 500, xx, xx[100] + 500), 
c( min( log( skeltimefed[-c(1:2), 3] ), log( skeltimepc[-c(1:2), 3] ), log( skeltimemm[-c(1:2), 3] ) ) - 1200, 
rep( max( log( skeltimefed[-c(1:2), 3] ), log( skeltimepc[-c(1:2), 3] ), log( skeltimemm[-c(1:2), 3] ) ) + 1200, 100), 
min( log( skeltimefed[-c(1:2), 3] ), log( skeltimepc[-c(1:2), 3] ), log( skeltimemm[-c(1:2), 3] ) ) - 1200 ), col = "lightgrey" )

abline(v = seq(4, 18, by = 1), col = "white", lwd = 2)
abline(h = seq(-4, 6, 1), col = "white", lwd = 2)
lines( log( n[-c(1:2)] ), log( skeltimefed[-c(1:2), 3] ), col = 3, lwd = 2)
points( log( n[-c(1:2)] ), log( skeltimefed[-c(1:2), 3] ), pch = 19, col = 3, cex = 1.2)
lines( log( n[-c(1:2)] ), log( skeltimepc[-c(1:2), 3] ), col = 2, lwd = 2)
points( log( n[-c(1:2)] ), log( skeltimepc[-c(1:2), 3] ), pch = 18, col = 2, cex = 1.2)
lines( log( n[-c(1:2)] ), log( skeltimemm[-c(1:2), 3] ), col = 1, lwd = 2)
points( log( n[-c(1:2)] ), log( skeltimemm[-c(1:2), 3] ), pch = 17, col = 1, cex = 1.2)

legend("topleft", c("FEDHC", "PCHC", "MMHC-2"), col = c(3:1), 
pch = c(19, 18, 17), lwd = rep(2, 3), cex = 1.2 , text.col = c(3:1) )

dev.off()


png(filename = paste("time_5_50.png", sep = ""), width = 6000, height = 5000, res = 700)

plot( log( n[-c(1:2)] ), log( timefed[-c(1:2), 3] ), type = "b", pch = 19, lwd = 2, col = 3, main = "50 variables",
ylim = c( min( log( timefed[-c(1:2), 3] ), log( timepc[-c(1:2), 3] ), log( timemm[-c(1:2), 3] ) ), 
max( log( timefed[-c(1:2), 3] ), log( timepc[-c(1:2), 3] ), log( timemm[-c(1:2), 3] ) ) ), 
xlab = "Sample size in log-scale", ylab = "Computational cost in log seconds", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq( min( log(n[-c(1:2)] ) ), max( log(n[-c(1:2)] ) ), length = 100)
polygon( c(xx[1] - 500, xx, xx[100] + 500), 
c( min( log( timefed[-c(1:2), 3] ), log( timepc[-c(1:2), 3] ), log( timemm[-c(1:2), 3] ) ) - 1200, 
rep( max( log( timefed[-c(1:2), 3] ), log( timepc[-c(1:2), 3] ), log( timemm[-c(1:2), 3] ) ) + 1200, 100), 
min( log( timefed[-c(1:2), 3] ), log( timepc[-c(1:2), 3] ), log( timemm[-c(1:2), 3] ) ) - 1200 ), col = "lightgrey" )

abline(v = seq(4, 18, by = 1), col = "white", lwd = 2)
abline(h = seq(-2, 8, 1), col = "white", lwd = 2)
lines( log( n[-c(1:2)] ), log( timefed[-c(1:2), 3] ), col = 3, lwd = 2)
points( log( n[-c(1:2)] ), log( timefed[-c(1:2), 3] ), pch = 19, col = 3, cex = 1.2)
lines( log( n[-c(1:2)] ), log( timepc[-c(1:2), 3] ), col = 2, lwd = 2)
points( log( n[-c(1:2)] ), log( timepc[-c(1:2), 3] ), pch = 18, col = 2, cex = 1.2)
lines( log( n[-c(1:2)] ), log( timemm[-c(1:2), 3] ), col = 1, lwd = 2)
points( log( n[-c(1:2)] ), log( timemm[-c(1:2), 3] ), pch = 17, col = 1, cex = 1.2)

legend("topleft", c("FEDHC", "PCHC", "MMHC-2"), col = c(3:1), 
pch = c(19, 18, 17), lwd = rep(2, 3), cex = 1.2 , text.col = c(3:1) )

dev.off()





n <- c(100, 500, 1000, 5000, 1e+4, 3e+4, 5e+4, 1e+5, 3e+5, 5e+5, 1e+6, 3e+6, 5e+6)

R <- 50
p <- 100
shdpc <- matrix(nrow = length(n), ncol = length(p) )
colnames(shdpc) <- paste("p=", p, sep = "")
rownames(shdpc) <- paste("n=", n, sep = "")
shdfed <- shdmm <- shdpc  ## SHD
timefed <- timepc <- timemm <- shdpc  ## times
ntestsfed <- ntestspc <- ntestsmm <- shdpc  ## number of tests
skeltimefed <- skeltimepc <- skeltimemm <- shdpc

  for ( i in 1:length(n) ) {  ## sample size
    for ( j in length(p) ) { ## number of variables
      st1 <- st2 <- st3 <- t1 <- t2 <- t3 <- shd1 <- shd2 <- shd3 <- test1 <- test2 <- test3 <- numeric(R)
      for (vim in 1:R) {
        m <- MXM::rdag2(n = n[i], p = p[j], nei = 5)
        G1 <- m$G
        g1 <- pcalg::dag2essgraph( as(G1, "graphNEL") )  ## turn the true DAG to an essential DAG
        x <- m$x ## data
        ## run the algorigthms
        #mod <- system.time( a1 <- bnlearn::mmhc( as.data.frame(x), restrict.args = list(alpha = 0.05, test = "zf") ) )
	  a1 <- pchc::fedhc(x, alpha = 0.05) 
        a2 <- pchc::pchc(x, alpha = 0.05)
	  a3 <- pchc::mmhc(x, alpha = 0.05)
        ## performance evaluation
	  st1[vim] <- a1$ini$runtime   ;    st2[vim] <- a2$ini$runtime  ;   st3[vim] <- a3$ini$runtime  ## runtimes
        t1[vim] <- a1$runtime   ;    t2[vim] <- a2$runtime ;   t3[vim] <- a3$runtime  ## runtimes
	  b1 <- as( bnlearn::amat(a1$dag), "graphNEL" )
	  b2 <- as( bnlearn::amat(a2$dag), "graphNEL" )
        b3 <- as( bnlearn::amat(a3$dag), "graphNEL" )
        shd1[vim] <- pcalg::shd(b1, g1)   ;  shd2[vim] <-  pcalg::shd(b2, g1)   ;  shd3[vim] <-  pcalg::shd(b3, g1)
        test1[vim] <- a1$ini$n.tests   ;   test2[vim] <- sum(a2$ini$n.tests)   ;   test3[vim] <- a3$ini$n.tests
      }  ## end for (vim in 1:R) {
	skeltimefed[i, j] <- mean(st1)
      skeltimepc[i, j] <- mean(st2)
	skeltimemm[i, j] <- mean(st3)
	  
      timefed[i, j] <- mean(t1)
      timepc[i, j] <- mean(t2)
      timemm[i, j] <- mean(t3)

      shdfed[i, j] <- mean(shd1)
      shdpc[i, j] <- mean(shd2)
      shdmm[i, j] <- mean(shd3)

      ntestsfed[i, j] <- mean(test1)
      ntestspc[i, j] <- mean(test2)
	ntestsmm[i, j] <- mean(test3)
	
      x <- NULL  
      save.image("sims_5_100.RData")
    }  ##  end  for ( j in 1:length(p) ) {
  }  ##  end  for ( i in 1:length(n) ) {



### SHD
png(filename = paste("shd_5_100.png", sep = ""), width = 6000, height = 5000, res = 700)

plot(log(n), shdfed[, 1], type = "b", pch = 19, lwd = 2, col = 3, main = "100 variables",
ylim = c( min(shdfed[, 1], shdpc[, 1], shdmm[, 1]), max(shdfed[, 1], shdpc[, 1], shdmm[, 1]) ), 
xlab = "Sample size in log-scale", ylab = "SHD", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(min(log(n)), max(log(n)), length = 100)
polygon( c(xx[1] - 100, xx, xx[100] + 100), 
c( min(shdfed[, 1], shdpc[, 1], shdmm[, 1]) - 100, rep(max(shdfed[, 1], shdpc[, 1], shdmm[, 1]) + 100, 100), 
min(shdpc[, 1], shdpc[, 1], shdmm[, 1]) - 100), col = "lightgrey" )

abline(v = seq(4, 16, by = 2), col = "white", lwd = 2)
abline(h = seq(10, 160, 4), col = "white", lwd = 2)
lines(log(n), shdfed[, 1], col = 3, lwd = 2)
points(log(n), shdfed[, 1], pch = 19, col = 3, cex = 1.2)
lines(log(n), shdpc[, 1], col = 2, lwd = 2)
points(log(n), shdpc[, 1], pch = 18, col = 2, cex = 1.2)
lines(log(n), shdmm[, 1], col = 1, lwd = 2)
points(log(n), shdmm[, 1], pch = 17, col = 1, cex = 1.2)

legend("topright", c("FEDHC", "PCHC", "MMHC-2"), col = c(3:1), 
pch = c(19, 18, 17), lwd = rep(2, 3), cex = 1.2 , text.col = c(3:1) )

dev.off()


## Number of CI tests


png(filename = paste("ntests_5_100.png", sep = ""), width = 6000, height = 5000, res = 700)

plot(log(n), ntestsfed[, 1], type = "b", pch = 19, lwd = 2, col = 3, main = "100 variables",
ylim = c( min(ntestsfed[, 1], ntestspc[, 1], ntestsmm[, 1]), max(ntestsfed[, 1], ntestspc[, 1], ntestsmm[, 1]) ), 
xlab = "Sample size in log-scale", ylab = "Number of tests", cex = 1.2, cex.lab = 1.2, cex.axis = 1.2)

xx <- seq(0.5 * min(log(n)), 3 * max(log(n)), length = 100)
polygon( c(xx[1] - 30000, xx, xx[100] + 30000), 
c( min(ntestsfed[, 1], ntestspc[, 1], ntestsmm[, 1]) - 30000, rep(max(ntestsfed[, 1], ntestspc[, 1], ntestsmm[, 1]) + 30000, 100), 
min(ntestspc[, 1], ntestspc[, 1], ntestsmm[, 1]) - 30000 ), col = "lightgrey" )

abline(v = seq(4, 16, by = 2), col = "white", lwd = 2)
abline(h = seq(0, 800000, 20000), col = "white", lwd = 2)
lines(log(n), ntestsfed[, 1], col = 3, lwd = 2)
points(log(n), ntestsfed[, 1], pch = 19, col = 3, cex = 1.2)
lines(log(n), ntestspc[, 1], col = 2, lwd = 2)
points(log(n), ntestspc[, 1], pch = 18, col = 2, cex = 1.2)
lines(log(n), ntestsmm[, 1], col = 1, lwd = 2)
points(log(n), ntestsmm[, 1], pch = 17, col = 1, cex = 1.2)

legend("topleft", c("FEDHC", "PCHC", "MMHC-2"), col = c(3:1), 
pch = c(19, 18, 17), lwd = rep(2, 3), cex = 1.2 , text.col = c(3:1) )

dev.off()
